package PureMessage::Redis::AdvanceConfig;

use strict;
use warnings;
use PureMessage::Config qw(expand_filename);
use base qw(PureMessage::Config::Validator Exporter);

our @EXPORT_OK = '%advance_config';
our %advance_config;

# Validation rules and default values:
my %conf = (
	spam_threshold        => ['#num', 9, {
								# 'unit' => 'number',
								# 'min'  => ?,
								# 'max'  => ?,
							 }],
	ham_threshold         => ['#num', 10,{
								# 'unit' => 'number',
								# 'min'  => ?,
								# 'max'  => ?,
							 }],
	spam_reset_count      => ['#num', 5, {
								# 'unit' => 'number',
								# 'min'  => ?,
								# 'max'  => ?,
							 }],
	ham_reset_count       => ['#num', 3, {
								# 'unit' => 'number',
								# 'min'  => ?,
								# 'max'  => ?,
							 }],
	db_cleanup_interval   => ['#num', 1, {
								# 'unit' => 'number',
								# 'min'  => ?,
								# 'max'  => ?,
							 }],

	delay_db_expiry_age   => ['#num', 180, {
								'unit' => 'days',
								'min'  => 90,
								'max'  => 365,
							 }],
	delay_db_min_count    => ['#num', 1000, {
								'unit' => 'no of records',
								'min'  => 100,
								'max'  => 10000,
							 }],
	delay_db_expiry_count => ['#num', 10000, {
								'unit' => 'no of records',
								'min'  => 1000000,
								'max'  => 1500000,
							 }],
);

refresh();

sub new {
	my $self = shift;
	my $p = ActiveState::Config->new(strict => 1, allow_includes => 1);
	PureMessage::Redis::AdvanceConfig->validate($p);
	$p->parse_file(expand_filename("etc/lua_args.conf"));
	$p->as_hash;
	$p;
}

sub refresh {
	%advance_config = new(undef)->as_hash;
	foreach my $key ( keys %advance_config) {
		my ($default, $condition) = @{$conf{$key}}[1..2];
		if ( %$condition ) {
			if ( exists $condition->{min} && ( $advance_config{$key} < $condition->{min} || $advance_config{$key} > $condition->{max} ) ) {
				$advance_config{$key} = $default;
			}
		}
	}
}

sub validate {
    my ($pkg, $p) = @_;

    # Add our validation rules to the parser.
    foreach my $key (keys %conf) {
        my ($type, $def) = @{$conf{$key}};
        $p->typedecl( $key, $type ) if (defined $type);
        $p->default(  $key, $def )  if (defined $def);
    }
}

1;
