
redis.replicate_commands()

-- time ./src/redis-cli --eval test5.2.2__.lua 9 10 25 3 2 1492303764
-- Get values from arguments. Sequence :	spam_threshold, ham_threshold, spam_reset_count, db_cleanup_interval, ham_reset_count, now/timeSinceEpoch

local spam_threshold		= tonumber(ARGV[1])
local ham_threshold			= tonumber(ARGV[2])
local spam_reset_count		= tonumber(ARGV[3])
local db_cleanup_interval	= tonumber(ARGV[4])
local ham_reset_count		= tonumber(ARGV[5])
local now					= tonumber(ARGV[6])

-- gets multiple fields from a hash as a dictionary
local hmget = function (key, ...)
	if next(arg) == nil then return {} end
	local bulk = redis.call('HMGET', key, unpack(arg))
	local result = {}
	-- Note: We are calling tonumber() on every value assuming all keys passed have numeric/time type of value
	for i, v in ipairs(bulk) do result[ arg[i] ] = tonumber(v) end
	return result
end

-- Process key records
local process = function(key)
	-- Spammy day formula: ((sc_today/mc_today)*100) > config(spam_threshold)
	-- Hammy day formula = ((((mc_today - sc_today)/mc_today)*100) > (100 - config(spam_threshold))) AND ((mc_today - sc_today) > config(ham_threshold))
	local r = hmget(key, 'sc_today', 'mc_today', 'sc', 'fs', 'hd', 'fso')
	local is_spammy_day	= ((r['sc_today']/r['mc_today'])*100 > spam_threshold)
	local is_hamy_day	= ( ( ((r['mc_today'] - r['sc_today'])/r['mc_today']) *100 ) > (100 - spam_threshold) ) and ( (r['mc_today'] - r['sc_today']) > ham_threshold )

	-- Now update sd_hd_str, incr sd or hd if required, reset sc_today and mc_today to 0
	local sd_hd_str = redis.call('HGET',key,'sd_hd_str')
	
	if string.len(sd_hd_str) >= 30 then
		sd_hd_str = string.sub(sd_hd_str, 2)
	end
	
	if (is_hamy_day) then
		sd_hd_str = sd_hd_str..'2'
	elseif(is_spammy_day) then
		sd_hd_str = sd_hd_str..'1'
	else
		sd_hd_str = sd_hd_str..'0'
	end
	
	-- update sd,hd . reset sc_today and mc_today
	local newStr  , hdCnt = string.gsub(sd_hd_str, "2","2")
	r['hd'] = hdCnt
	local newStr2 , sdCnt = string.gsub(sd_hd_str, "1","1")
	r['sd'] = sdCnt
	redis.call('HMSET',key, 'sd_hd_str', sd_hd_str, 'sc_today', 0, 'mc_today', 0, 'hd', hdCnt, 'sd', sdCnt)
	
	--  if ( (sc > config(spam_reset_count)) AND (fs <= time - 3 * ((24*3600) * config(db_cleanup_interval))) ) OR ( (hammy_day_count <= config(ham_reset_count)) AND (fso < now - (30*24*3600)) )
	-- 		Set fs, mc, sc fields to zero
	--		Increment nr  
	if ( ( ( r['sc'] > spam_reset_count ) and ( r['fs'] <= (now - (3*((24*3600)*db_cleanup_interval)) ) ) )
		or
		( ( r['hd'] <= ham_reset_count ) and (r['fso'] < (now - (30 * 24 * 3600 )) ) )
	   )then
		redis.call( 'HMSET',  key, 'fs', 0, 'mc', 0, 'sc', 0 )
		redis.call( 'HINCRBY',key, 'nr', 1)
	end
	return key
end

local cursor = "0"
local count  = 0

repeat
	local r = redis.call("SCAN", cursor, "MATCH", "*", "COUNT", 100)
	-- local r = redis.call("SCAN", cursor, "MATCH", "192.168.156.253", "COUNT", 100)
	cursor = r[1]
	local k = r[2]

	for i = 1, #k do
		-- print( 'k[i]', k[i] )
		-- Database have all keys in ip address format, expect key db_age
		if ( k[i] ~= 'db_age') then
			local result = process(k[i])
		end
	end
	count = count + #r[2]
until cursor == "0"


